#!/usr/bin/env python
# Read roots from file, generate postscript:
#
#   zcat roots{6,5,4,3,2,1}-[123].txt.gz | ./plotfromfile.py | gv --scale=10 -
#   zcat roots{6,5,4,3,2,1}-[123].txt.gz | ./plotfromfile.py > roots6-3.ps
#
# The degrees are ordered so that high degree points are painted first.

import sys

import psyco
psyco.full()

pshead = """\
%!PS-Adobe-2.0
%
% Global variables that are easy to tweak:
%
% An additional overall scale factor:
/stretch 1.8 def
%
% An additional scale factor just for radius of dots:
/radstretch 0.0009 def
% Currently the dots are different sizes depending on the degree,
% but this can be adjusted by changing the definition of S below.
%
% work in inches:
72 72 scale
%
% center on letter-size page:
4.25 5.5 translate
%
90 rotate
%
% Take into account overall scale factor:
stretch stretch scale
%
% Procedures defined below:
%
%  r g b C   sets the colour
%  rad   R   sets the radius
%  d h   S   sets colour/radius based on degree d, height h of poly
%  x y   D   draw a solid circle at (x,y) using current colour/radius
%  x y   F   draws four dots at (+-x, +-y)
%  x y   E   draws eight dots: four like D, and four more
%            corresponding to 1/(x+iy)
%
% This is the routine that figures out the colour and dotsize,
% based on degree and height of polynomial.  Out of all the
% procedures, it's the most interesting to play with:
/r1 1.0 def /g1 0.4 def /b1 0.4 def  % deg 6 colour
/r2 0.0 def /g2 0.0 def /b2 0.0 def  % deg 1 colour
/rads [ 1.9 1.7 1.5 1.3 1.0 0.47 0.12 ] def
/S { 
   pop                          % ignore height for now
   /deg exch def                % store degree
   rads deg get R               % set radius based on deg
   /t deg 1 sub 5 div def       % store t=(deg-1)/5
   /omt t 1 sub def             % store omt=1-t
   r1 t mul r2 omt mul add      % linearly interpolate colours
   g1 t mul g2 omt mul add
   b1 t mul b2 omt mul add
   C
   } def
/C { setrgbcolor } def
/R { radstretch mul /r exch def } def
/D { r newpath 0 360 arc eofill } def
/F { 1 index neg 1 index  % x y -x y
     2 copy neg           % x y -x y -x -y
     D D                  % draw two, leaves: x y
     2 copy neg           % x y x -y
     D D                  % draw two
   } def
/E { 2 copy               % x y x y
     F                    % draw +-x +-y
     /y exch def
     /x exch def          % stack empty, x and y saved
     /n x x mul y y mul add def % n contains x^2 + y^2
     n 0.001 ge                 % if n > 0.001:
       { x n div  y n div F }   %   draw +-x/n +-y/n
     if
   } def
%
% To figure out the above, look at
%   http://www-cdf.fnal.gov/offline/PostScript/PLRM2.pdf

"""

def psinit():
    print pshead

prevorder = -1
prevheight = -1
def psdraw(rt, order, height):
    global prevorder
    global prevheight
    x = rt.real
    y = rt.imag
    if order != prevorder or height != prevheight:
        prevorder = order
        prevheight = height
        print order, height, 'S'
    print sht(x), sht(y), 'E'

def sht(x):
    '''Return a short string representation of a float.'''
    # %.4f includes exactly four digits to the right of the
    # decimal point, i.e. it specifies the absolute precision,
    # which is what we care about for plotting.  The rstrip
    # removes unneeded trailing 0's and decimal point.
    # (rstrip('0.') changes '0.0000' to ''.)
    # (%.4g gives four significant digits, which is silly
    # for really small numbers.)
    return ('%.4f' % (x,)).rstrip('0').rstrip('.')

def pls():
    for line in sys.stdin:
        left, right = line.split(';')
        coeffs = map(int, left.split())
        order = len(left.split())-1
        height = max(map(abs, coeffs))
        rts = map(complex, right.split())
        for rt in rts:
            if rt.imag >= 0:
                psdraw(rt, order, height)

if __name__ == '__main__':
    psinit()
    pls()
