"""
Various ways of taking Cartesian product of a list of lists.
The answer is returned as a list of tuples or lists, or sometimes
as a generator of tuples or lists.

Best generator:  xcartprod  26.3ms
Best list:       cartprod   30.6ms

But note that for my polynomial roots code, xcartprod2, 3, and 7
are all better than xcartprod, even though they are *much* slower
for the [range(3)]*10 test.  Weird.

For doing powers:   (negligible benefit, but cleaner to call)

Best generator:  xcartpower     26.3ms
Best list:       cartpower      30.4ms
   
If you need to save state:

                 yintervalpower 88ms

For example:

    cartprod([[1,2], [3,4,5]]) gives
       [(1,3), (2,3), (1,4), (2,4), (1,5), (2,5)]
"""

import operator

# Timings are for:
#   for x in prodfunction(seqs): pass
# with
#   seqs = [range(3)]*10
# Or:
#   for x in powerfunction(seq, 10): pass
# with
#   setq = range(3)
# Designed to favour generators.

# A version whose state can be easily pickled.
# Only handles powers of numerical intervals like range(mn,mx+1,step).
# 
# Increments first spot instead of last, and
# unrolls part of loop to handle most common case quickly.
# Usage:
# cur = [mn]*num
# while True:
#     ...use cur...
#     if not yintervalpower(cur, mn, mx, step):
#         break
# 88ms
# Prefix "y" because it is sort of like a generator.
def yintervalpower(cur, mn, mx, step=1):
    '''Increment cur to "next" element of cartesian product
    range x range x ... x range.  Given a tuple (mn, mx, step),
    range is { x | x = mn + k*step, k >= 0 and x <= mx }.
    (In particular, mn, mx and step can be floats.  Be careful
    about endpoints.)  step defaults to 1.
    cur is a list which is updated in place.
    The number of factors is len(cur).
    cur[0] is updated most frequently.'''

    # Parsing a tuple each time is very slow.
    #mn = 0; step = 1
    #if isinstance(rng, (int, float)):
    #    mx = rng
    #else:
    #    if len(rng) == 1:
    #        (mx,) = rng
    #    elif len(rng) == 2:
    #        (mn, mx) = rng
    #    elif len(rng) == 3:
    #        (mn, mx, step) = rng
    #    else:
    #        raise ValueError, 'Expected int, float or tuple of such of len <= 3.'

    if cur[0] < mx:
        cur[0] += step
        return True
    num = len(cur)
    pos = 0
    while cur[pos] == mx:
        cur[pos] = mn
        pos += 1
        if pos >= num: return False
    cur[pos] += step
    return True

# Tried altering a list/array in place,
# and yielding the "same" list/array each time.
# Assumes the seq is range(mn, mx+1).
# Not bad...
# 60.7ms
def xintervalpower(mn, mx, num):
    if num == 0:
        yield [()]
        return
    cur = [mn] * num
    while True:
        yield cur
        pos = num-1
        while cur[pos] == mx:
            cur[pos] = mn
            pos -= 1
            if pos < 0: return
        cur[pos] += 1

# Based on Tim Peter's cartprod below, but assumes all seqs the same.
# Improved from 36 ms to 30 ms by replacing sprayb stuff with
# naive list comprehension (for my [range(3)]*10 data set).
# 30 ms
def cartpower(seq, num):
    '''Cartesian product seq x seq x ... x seq with num factors.'''
    if num == 0:
        return [()]
    # This block is not logically necessary, but gives a big speedup:
    if num == 1:
        return map(lambda i: (i,), seq)
    n = num/2
    a = cartpower(seq, n)
    b = xcartpower(seq, num-n)
    return [x+y for x in b for y in a]

# Generator version of xcartpower.
# 26.5 ms!
def xcartpower(seq, num):
    '''Cartesian product seq x seq x ... x seq with num factors.'''
    if num == 0:
        yield ()
        return
    # This block is not logically necessary, but gives a big speedup:
    if num == 1:
        for x in seq:
            yield (x,)
        return
    n = num/2
    a = cartpower(seq, n)
    for x in xcartpower(seq, num-n):
        for y in a:
            yield x+y

# This version was written by Tim Peters and is somewhat faster
# than cartprod2, especially for large numbers of small lists.
# Improved from 37 to 35ms by replacing sprayb stuff with
# naive list comprehension and using xcartprod for half of
# recursion (for my [range(3)]*10 data set).
# 30.3 ms!
def cartprod(seqs):
    n = len(seqs)
    if n == 0:
        return [()]
    if n == 1:
        return map(lambda i: (i,), seqs[0])
    # find good splitting point
    prods = []
    prod = 1
    for x in seqs:
        prod = prod * len(x)
        prods.append(prod)
    for i in range(n):
        if prods[i] ** 2 >= prod:
            break
    n = min(i + 1, n - 1)
    a = cartprod(seqs[:n])
    return [x+y for y in xcartprod(seqs[n:]) for x in a]
    #b = cartprod(seqs[n:])
    #sprayb = []
    #lena = len(a)
    #for x in b:
    #    sprayb[len(sprayb):] = [x] * lena
    #return map(operator.add, a * len(b), sprayb)

# Generator version of Tim Peters' cartprod above.
# 26.3ms!
def xcartprod(seqs):
    n = len(seqs)
    if n == 0:
        yield ()
        return
    if n == 1:
        for x in seqs[0]:
            yield (x,)
        return
    # find good splitting point
    prods = []
    prod = 1
    for x in seqs:
        prod = prod * len(x)
        prods.append(prod)
    for i in range(n):
        if prods[i] ** 2 >= prod:
            break
    n = min(i + 1, n - 1)
    a = cartprod(seqs[:n])
    for y in xcartprod(seqs[n:]):
        for x in a:
            yield x+y

# 62 ms
def cartprod2(Lists):
  import operator
  if Lists:
    result = map(lambda I: (I,), Lists[0])

    for list in Lists[1:]:
      curr = []
      for item in list:
        new = map(operator.add, result, [(item,)]*len(result))
        curr[len(curr):] = new
      result = curr
  else:
    result = []

  return result

# This is xcombine from the python cookbook, with inner function
# removed.
# 269 ms
def xcartprod2(seqs, comb=[]):
    '''A generator which returns combinations of argument sequences
for example xcombine4([(1,2),(3,4)]) returns a generator yielding
[1,3], [1,4], [2,3], [2,4].'''
    if seqs:
        for item in seqs[0]:
            newcomb=comb+[item]
            for item in xcartprod2(seqs[1:], newcomb):
                yield item
    else:
        yield comb

# producing tuples is faster than producing lists.
# And shortcutting for len(seqs) == 1 helps a lot.
# 180 ms
def xcartprod3(seqs, comb=()):
    '''A generator which returns combinations of argument sequences
for example xcartprod3([(1,2),(3,4)]) returns a generator yielding
[1,3], [1,4], [2,3], [2,4].'''
    if not seqs:
        yield comb
        return
    # This block is not logically necessary, but gives a big speedup:
    if len(seqs) == 1:
        for x in seqs[0]: yield comb+(x,)
        return
    for item in seqs[0]:
        newcomb=comb+(item,)
        for item in xcartprod3(seqs[1:], newcomb):
            yield item

# Raymond Hettinger
# 165ms
def cartprod3(args):
    ans = [()]
    for arg in args:
        ans = [ list(x)+[y] for x in ans for y in arg]
    return ans


# 103ms
def cartprod4(x):
    if x == []: return [[]]
    return [[z]+y for y in cartprod4(x[1:]) for z in x[0]]


# 73ms
def cartprod5(seqs):
    return reduce(lambda X1, X2: [ one+(two,) for one in X1 for two in X2],
                  seqs, [()] )

# Using a list of indices was horribly slow.
# 266 ms
def xcartprod7(seqin):
    num = len(seqin)
    maxes = [len(s)-1 for s in seqin]
    cur = [0] * num
    rng = range(num)
    while True:
        yield [(seqin[i])[cur[i]] for i in rng]
        pos = num-1
        while cur[pos] == maxes[pos]:
            cur[pos] = 0
            pos -= 1
            if pos < 0: return
        cur[pos] += 1

# The rest of this isn't my code, and wasn't used in my testing. jdc
if __name__ == "__main__":
  import time
  import random
  import math
  import operator

  N = 1e4   # Approximate number of items in the permutations

  funcs = [permute, permute2]

  Times = {}
  map(operator.setitem, [Times]*len(funcs), funcs, [0]*len(funcs))

  print 'Testing small numbers of large lists...'
  for i in range(10):
    print '.',
    listLen = random.randint(2,5)
    TheList = []
    eachListLen = int(math.pow(N, 1.0/listLen) + 0.5)
    for ix in range(listLen):
      TheList.append([0] * eachListLen)

    for func in funcs:
      tic = time.clock()
      result = apply(func, (TheList,))
      toc = time.clock()
      Times[func] = Times[func] + toc-tic

  print
  for func in funcs:
    print '%s: %g' % (func.__name__, Times[func])

  map(operator.setitem, [Times]*len(funcs), funcs, [0]*len(funcs))
  print 'Testing large numbers of small lists...'

  for i in range(10):
    print '.',
    listLen = random.randint(20,30)
    TheList = []
    eachListLen = N
    while eachListLen > 1:
      TheList.append([0,0])
      eachListLen = eachListLen / 2
    for ix in range(len(TheList), listLen):
      TheList.append([0])

    for func in funcs:
      tic = time.clock()
      result = apply(func, (TheList,))
      toc = time.clock()
      Times[func] = Times[func] + toc-tic
    
  print
  for func in funcs:
    print '%s: %g' % (func.__name__, Times[func])

  print 'Checking correctness...'
  def ctest(func):
    def listeq(L1,L2):
      L1.sort()
      L2.sort()
      return L1==L2

    try:
      assert apply(func, ([],)) == []
      assert apply(func, ([[1,2,3]],)) == [(1,),(2,),(3,)]
      assert apply(func, (["a", "b", "c", "d"],)) == [('a','b','c','d')]
      assert listeq(apply(func, ([[1,2,3], "ab"],)), 
                                        [(1,'a'),(1,'b'),
                                        (2,'a'),(2,'b'),
                                        (3,'a'),(3,'b')])
      assert apply(func, ([range(100), range(0)],)) == []
      assert listeq(apply(func, ([range(4), range(2), range(1), range(2)],)),
          [(0, 0, 0, 0), (0, 0, 0, 1), (0, 1, 0, 0), (0, 1, 0, 1),
           (1, 0, 0, 0), (1, 0, 0, 1), (1, 1, 0, 0), (1, 1, 0, 1),
           (2, 0, 0, 0), (2, 0, 0, 1), (2, 1, 0, 0), (2, 1, 0, 1),
           (3, 0, 0, 0), (3, 0, 0, 1), (3, 1, 0, 0), (3, 1, 0, 1)])
    except:
      print 'Correctness test FAILED for %s' % func.__name__
      return
    print 'Correctness test PASSED for %s' % func.__name__

  for func in funcs:
    ctest(func)
